# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import os
import psutil
import re

from cmf.monitor import schema

_FD_MAX_RE = re.compile('Max open files\s+\d+\s+(\d+)\s+files')
_SWAP_OUT_RE = re.compile('pswpout (\d+)')

_VMSTAT_PARSE_FAILED = False

def get_fd_max(pid, logger=None):
  if pid is None:
    return None
  f = None
  try:
    try:
      f = open('/proc/%d/limits' % pid, 'r')
      for line in f:
        m = _FD_MAX_RE.match(line)
        if m:
          return int(m.groups()[0])
      if logger is not None:
        logger.error("Could not parse limits for for process %d" % pid)
      return None
    except (Exception), e:
      if logger is not None:
        logger.error("Failed to read file descriptor max for process %d: %s" %
                     (pid, e))
      return None
  finally:
    if f is not None:
      f.close()

def get_fd_count(pid, logger=None):
  if pid is None:
    return None
  try:
    return len(os.listdir("/proc/%d/fd/" % pid))
  except (Exception), e:
    if logger is not None:
      logger.error("Failed to get file descriptor count for process %d: %s" %
                   (pid, e))
    return None

def get_swap_out(logger=None):
  global _VMSTAT_PARSE_FAILED

  f = None
  try:
    try:
      f = open('/proc/vmstat', 'r')
      for line in f:
        m = _SWAP_OUT_RE.match(line)
        if m:
          _VMSTAT_PARSE_FAILED = False
          return int(m.groups()[0])
      if logger is not None and not _VMSTAT_PARSE_FAILED:
        logger.error("Could not parse pswpout from /proc/vmstat")
        _VMSTAT_PARSE_FAILED = True
        return None
    except (Exception):
      if logger is not None:
        logger.error("Failed to get pages swapped out: ", exc_info=True)
        _VMSTAT_PARSE_FAILED = True
      return None
  finally:
    if f is not None:
      f.close()

def add_proc_metrics(update, pid, logger=None, add_mem_metrics=False):
  # For the rest of the metrics, we require a pid, so we're done if we were not
  # given one.
  if pid is None:
    return

  try:
    fd_max = get_fd_max(pid, logger)
    if fd_max is not None:
      update.add_metric(schema.FD_MAX, fd_max)

    fd_count = get_fd_count(pid, logger)
    if fd_count is not None:
      update.add_metric(schema.FD_OPEN, fd_count)

    process = psutil.Process(pid)
    cpu_times = process.get_cpu_times()
    if cpu_times is not None:
      update.add_metric(schema.TOTAL_CPU_USER, cpu_times.user)
      update.add_metric(schema.TOTAL_CPU_SYSTEM, cpu_times.system)

    if add_mem_metrics:
      rss, vms = process.get_memory_info()
      update.add_metric(schema.MEM_RSS, rss)
      update.add_metric(schema.MEM_VIRTUAL, vms)

  except (Exception), e:
      if logger is not None:
        logger.error("Failed to get process metrics %d: %s" %
                     (pid, e))
      return None
